#include <gtk/gtk.h>
#include <stdlib.h>
#include "menu.h"
#include "toolbar.h"
#include "dead.xpm"
#include "red.xpm"
#include "green.xpm"
#include "blue.xpm"
#include "yellow.xpm"

/* For detecting som callbacks and their coordinates. */
struct cord {
  gint x, y;
};

/* The actual playtable. */
GtkWidget *playtable[MAX_POPULATIONS][MAX_POPULATIONS];

/* The main window. */
GtkWidget *window;

/* For drawing. */
GdkBitmap *mask;
GdkPixmap *dead, *red, *green, *blue, *yellow;
GtkStyle *style;
Population *pop, *old, *oldold, *tmp;

/* Default specie. */
extern Species set_specie;

/* Done only before Run or between Stop and Run. */
gint update_specie(GtkWidget *widget, struct cord *data)
{
  /* Well, one have clicked in a cell. */
  (*pop)[data->x][data->y] = set_specie;
  
  switch(set_specie) {
  case RED :
    gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(widget)->child), red, mask);
    break;
  case GREEN : 
    gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(widget)->child), green, mask);
    break;
  case BLUE :
    gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(widget)->child), blue, mask);
    break;
  case YELLOW : 
    gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(widget)->child), yellow, mask);
    break;
  default :
    gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(widget)->child), dead, mask);
    break;
  }
  return 0;
}

/* Initialization function. Initializes the drawing variables. */
void initialize_specie_sort(void)
{
  dead = gdk_pixmap_create_from_xpm_d(window->window,  &mask,
				      &style->bg[GTK_STATE_NORMAL],
				      dead_xpm);
  
  red = gdk_pixmap_create_from_xpm_d(window->window,  &mask,
				     &style->bg[GTK_STATE_NORMAL],
				     red_xpm);
  green = gdk_pixmap_create_from_xpm_d(window->window,  &mask,
				       &style->bg[GTK_STATE_NORMAL],
				       green_xpm);
  blue = gdk_pixmap_create_from_xpm_d(window->window,  &mask,
				      &style->bg[GTK_STATE_NORMAL],
				      blue_xpm);
  yellow = gdk_pixmap_create_from_xpm_d(window->window,  &mask,
					&style->bg[GTK_STATE_NORMAL],
					yellow_xpm);
}

/* Initializes the playtable. */
GtkWidget *initialize_playtable(void)
{
  GtkWidget *vbox, *hbox, *pixmapwid;
  gint i, j;
  
  struct cord *xy;
  
  vbox = gtk_vbox_new(FALSE, 0);
  
  for(i=0; i < MAX_POPULATIONS; i++) {
    
    hbox = gtk_hbox_new(FALSE, 0);
    
    for(j=0; j < MAX_POPULATIONS; j++) {
      playtable[i][j] = gtk_button_new();

      xy = malloc(sizeof(xy)); /* For every new cell, one want an unique   */
      xy->x = i;               /* coordinate to send to the event handler. */
      xy->y = j;
      
      /* A pixmap widget to contain the pixmap */
      pixmapwid = gtk_pixmap_new(dead, mask);
      gtk_container_border_width(GTK_CONTAINER(playtable[i][j]), 0);
      gtk_container_add(GTK_CONTAINER(playtable[i][j]), pixmapwid);
      gtk_widget_show(pixmapwid);
      
      gtk_signal_connect(GTK_OBJECT (playtable[i][j]), "clicked", 
			 GTK_SIGNAL_FUNC (update_specie),(gpointer) xy);
      
      gtk_box_pack_start(GTK_BOX (hbox), playtable[i][j], FALSE, FALSE, FALSE);
      gtk_widget_show(playtable[i][j]);
    }
    gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, FALSE);
    gtk_widget_show(hbox);
  }
  return vbox;
}

/* Makes the main window and standard event handlier. */
GtkWidget *make_window(GtkWidget *widget)
{
  widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  
  gtk_window_set_title (GTK_WINDOW (widget), "gLife v0.05 beta");
  
  gtk_signal_connect (GTK_OBJECT (widget), "destroy", 
		      GTK_SIGNAL_FUNC (gtk_exit), NULL);
  
  gtk_signal_connect (GTK_OBJECT (widget), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_exit), NULL);
  
  gtk_container_border_width (GTK_CONTAINER (widget), 0);
  
  gtk_widget_realize(widget);
  
  return widget;
}

int main (int argc, char *argv[])
{
  GtkWidget *vbox, *play_table, *menu, *toolbar;

  gtk_init(&argc, &argv);

  /* Makes the main window with signals and everything... */
  window = make_window(window);
  
  /* For use when drawing pixmaps. */
  style = gtk_widget_get_style(window);
  
  /* Draw a totaly dead population */
  initialize_specie_sort();

  /* Initialize a dead population in the population matrix and some other
     (old, oldold for now) dead population. This will later be used for
     detecting infinite loops in the game.
   */
  if ((pop = malloc(sizeof(*pop))) != NULL) {
    tmp = malloc(sizeof(*tmp));
    old = malloc(sizeof(*old));
    oldold = malloc(sizeof(*oldold));
    pop = initialize_dead_population(pop);
    old = initialize_dead_population(old);
  }
  else {
    g_print("Error:\n\tAn unexpected error occured during initialization.\n");
    g_print("\tThe cause if probably \"out of memory\".\n");
    exit(1);
  }
  
  /* To pack stuff in. */
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (window), vbox);
  
  
  /* Make a menu + toolbar and put it in the vbox. */
  make_menu(window, &menu);
  gtk_box_pack_start(GTK_BOX (vbox), menu, FALSE, FALSE, 0);
 
  make_toolbar(window, &toolbar);
  gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
 
  /* Make and initialize the playtable. */
  play_table = initialize_playtable();
  gtk_container_add(GTK_CONTAINER (vbox), play_table);
  
  /* Show the widgets. */
  gtk_widget_show(menu); 
  gtk_widget_show(toolbar);
  gtk_widget_show(play_table);
  gtk_widget_show(vbox);
  gtk_widget_show(window);
  
  /* Well, we're ready to run. */
  gtk_main();
  
  return 0;
  
}
