#include <stdlib.h>
#include <unistd.h>
#include "life.h"

/* Defined in gLife.c */
extern Population *pop, *old, *oldold, *tmp;
extern GtkWidget *playtable[MAX_POPULATIONS][MAX_POPULATIONS];
extern GdkPixmap *dead, *red, *green, *blue, *yellow;
extern GdkBitmap *mask;

/* CHECK till be implemented later, if one will run the simulation without
   checking for infinite loops in the game. */
gint CHECK = TRUE;       /* If we want to find infinite loops in the game. */
gint RUN = FALSE;        /* As long as TRUE, we run the simulation. */
guint32 interval = 100;  /* Run the simulation every interval millisec. */

Population *initialize_dead_population (Population *p)
{
  gint i=0, j=0; /* Loop variables. */

  for(i=0; i < MAX_POPULATIONS; i++) 
    for(j=0; j < MAX_POPULATIONS; j++)
      (*p)[i][j] = DEAD; /* Everyone is dead at this point. */
  

  return p; /* Return a pointer to the population matrix. */
}

gint is_alive(Species S)      /* If the Population is alive or not. */
{
  return (S != DEAD);
}

gint neighbours_alive (Population *p, gint i, gint j)
{                                   /* Checks how many neighboyrs are alive. */
  gint n = 0;
  gint i1, i2, j1, j2, loopi, loopj;


  /* Implement some kind of over-the-edge later. */

  /*  if(i==0)
      i1 = MAX_POPULATIONS-1;
      else 
      i1 = i-1;
      
      if(i==(MAX_POPULATIONS-1))
      i2 = 0;
      else
      i2 = i+1;
      
      if(j==0)
      j1 = MAX_POPULATIONS-1;
      else
      j1= j-1;
      
      if(j==(MAX_POPULATIONS-1))
      j2 = 0;
      else
      j2 = j+1;
  */

  j1 = MAX(j-1, 0);                 /* Done to avoid problems at the egde... */
  j2 = MIN(j+1, MAX_POPULATIONS-1); /* Will make this go over the edge later */
  i1 = MAX(i-1, 0);
  i2 = MIN(i+1, MAX_POPULATIONS-1);

  for(loopi = i1; loopi <= i2; loopi++) 
    for(loopj = j1; loopj <= j2; loopj++)
      if (is_alive((*p)[loopi][loopj])) 
	n += 1;
  
  if (is_alive((*p)[i][j])) 
    return (n-1);       /* Number of neighbours alive in the area if */
  else                  /* the specie in the given cell is alive.    */
    return n;
}

Species dominating_neighbour(Population *p, gint i, gint j)
{    /* Checks which specie which is domination the area around a given cell */
  gint R = 0, G = 0, B = 0, Y = 0, Max_Nr_Pop = 0;
  gint i1, i2, j1, j2, loops, loopt;

  j1 = MAX(j-1, 0);
  j2 = MIN(j+1, MAX_POPULATIONS-1);
  i1 = MAX(i-1, 0);
  i2 = MIN(i+1, MAX_POPULATIONS-1);

  for(loops = i1; loops <= i2; loops++)
    for(loopt = j1; loopt <= j2; loopt++)
      if (is_alive((*p)[loops][loopt])) {       /* We don't need to check */
	if (!((loops == i) && (loopt == j))) {  /* DEAD cells, do we?     */
	  switch ((*p)[loops][loopt]) {
	  case RED : 
	    R += 1;
	    break;
	  case GREEN : 
	    G += 1;
	    break;
	  case BLUE : 
	    B += 1;
	    break;
	  case YELLOW : 
	    Y += 1;
	    break;
	  default : 
	    g_print("Error:\n\tAn unexpexted error have occured while\n");
	    g_print("\tcalculating the dominating neighbour.\n");
	    exit(1); 
	    break;
	  }
	}
      }

  Max_Nr_Pop = MAX(MAX(R,G),MAX(B,Y));

  if (Max_Nr_Pop == 1) /* Well...we have thre different species. */
    return DEAD;
  else if (Max_Nr_Pop == R)
    return RED;
  else if (Max_Nr_Pop == G)
    return GREEN;
  else if (Max_Nr_Pop == B)
    return BLUE;
  else if (Max_Nr_Pop == Y)
    return YELLOW;
  else {
    g_print("Error:\n\tAn unexpected error occured while counting ");
    g_print("the dominating neighbour.\n");
    exit(1);
  }
}

Species next(Population *p, gint i, gint j)
{
  /* How many neighbours that are alive. */
  gint alive = neighbours_alive(p,i,j); 

  if (is_alive((*p)[i][j])) {     /* The is game rules */
    if (alive == 2 || alive == 3)
      return (*p)[i][j];          /* It will survive one more generation */
    else
      return DEAD;                /* It will die. */
  }
  else {
    if (alive == 3)               /* A new one is born. */
      return dominating_neighbour(p, i, j); /* The dominating neighbour */
    else                                    /* takes the place. */
      return DEAD;   /* It stays DEAD. */
  }
}

gint copy_population(Population *p, Population *q)
{
  gint i=0, j=0; /* Loop variables. */
  
  for(i=0; i < MAX_POPULATIONS; i++)
    for(j=0; j < MAX_POPULATIONS; j++)
      (*p)[i][j] = (*q)[i][j];
  
  return 0;
}
  
Population *next_generation(Population *p)
{
  gint i=0, j=0;

  for(i=0; i < MAX_POPULATIONS; i++)
    for(j=0; j < MAX_POPULATIONS; j++)
      (*tmp)[i][j] = next(p, i, j);

  copy_population(p, tmp); /* I know it isn't pretty. */

  return p;  /* The new generation. */
}

/* Will be used later. */
gint only_one_left(Population *p)
{
  Species s = DEAD;
  gint i=0, j=0;

  for(i=0; i < MAX_POPULATIONS; i++)
    for(j=0; j < MAX_POPULATIONS; j++) {
      if(((*p)[i][j] != DEAD) && ((*p)[i][j] != s)) {
	if(s == DEAD) 
	  s = (*p)[i][j];
	else
	  return FALSE;
      }
    }
  return TRUE;
}

gint population_cmp(Population *p, Population *q)
{
   gint i=0, j=0;
  
  for(i=0; i < MAX_POPULATIONS; i++)
    for(j=0; j < MAX_POPULATIONS; j++)
      if((*p)[i][j] != (*q)[i][j])
	return FALSE;
  
  return TRUE;
} 

gint draw_generation(Population *op, Population *np)
{
  gint i=0, j=0; /* Loop variables. */

  for(i=0; i < MAX_POPULATIONS; i++) {
    for(j=0; j < MAX_POPULATIONS; j++) {
      if((*op)[i][j] != (*np)[i][j]) { /* Just have to redraw the changes, */ 
	switch((*np)[i][j]) {          /* not everything.                  */
	case RED :
	  gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(playtable[i][j])->child), 
			 red, mask);
	  break;
	case GREEN :
	  gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(playtable[i][j])->child), 
			 green, mask);
	  break;
	case BLUE :
	  gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(playtable[i][j])->child), 
			 blue, mask);
	  break;
	case YELLOW :
	  gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(playtable[i][j])->child), 
			 yellow, mask);
	  break;
	case DEAD :
	  gtk_pixmap_set(GTK_PIXMAP(GTK_BUTTON(playtable[i][j])->child), 
			 dead, mask);
	  break;
	default : 
	  g_print("Error:\n");
	  g_print("\tError when drawing %i playtable[%i][%i].\n",
		  (*np)[i][j], i, j); 
	  exit(1);
	  break; 
	}
      }
    }
  }
  return TRUE;
}

gint life(gpointer data)
{
  /* For detecting some of the infinite loops. Should be more then two, 
     like fifety or something. */
  copy_population(old, pop);
  
  /* Calculate the next generation. */
  pop = next_generation(pop);

  /* Draw the population. */
  draw_generation(old, pop);

  /* Check if we found any infinite loops. */
  /* Will implement an array of Population later (like 50 elements). */
  if(CHECK)
    if(population_cmp(oldold, pop) || population_cmp(old, pop))
      RUN = FALSE;
  copy_population(oldold, old);     

  return RUN; /* As long as TRUE, the engine will run. */
}

void run_life(GtkWidget *widget, gpointer data)
{
  RUN = TRUE; /* If one are here, one definately want to run the simulation 
		 until the end or until one say different. */

  /* for loop-detecting. Will be more advanced later. */
  if(CHECK)
    copy_population(oldold, old);

  /* Run the simulation every interval milliseconds. */
  gtk_timeout_add(interval, life, NULL);

}

gint step_life(GtkWidget *widget, gpointer data)
{
  /* We don't want to run the simulation until end, just once. */

  old = initialize_dead_population(old);

  if(CHECK)
    copy_population(oldold, old);

  life(FALSE);

  return 0;
}

gint step_life_back(GtkWidget *widget, gpointer data)
{
  /* Just draw it at this time, don't actually change the playtable!
     This will be changed later.
   */
  draw_generation(pop, old);

  return 0;
}

void stop_life(GtkWidget *widget, gpointer data)
{
  /* Well, we don't want to stop the simulation. */
  RUN = FALSE;
}

void clear_table(GtkWidget *widget, gpointer data)
{
  gint i=0, j=0; /* loop-variable. */

  for(i=0; i < MAX_POPULATIONS; i++)
    for(j=0; j < MAX_POPULATIONS; j++){
      (*pop)[i][j] = DEAD;
      (*old)[i][j] = RED;
    }

  draw_generation(old, pop);

}
