#include <gtk/gtk.h>
#include <stdlib.h>

#define MAX_POPULATIONS 20  /* The max height and width of the matrix. */

/* The Different Species that may exist. */
typedef enum
{
  DEAD,
  RED,
  GREEN,
  BLUE,
  YELLOW
} Species;

/* The population matrix. */
typedef Species Population[MAX_POPULATIONS][MAX_POPULATIONS];

/* Initializes a completely dead population. */
Population *initialize_dead_population (Population *p);

/* Checks whether a specie is dead or not. */
gint is_alive(Species S);

/* Checks how many species which are alive in a given area. */
gint neighbours_alive (Population *p, gint i, gint j);

/* Check which specie which is domination a given area. */
Species dominating_neighbour(Population *p, gint i, gint j);

/* Calculates the next specie in a matrix-cell. */
Species next(Population *p, gint i, gint j);

/* Calcualtes a completely new generation. */
Population *next_generation(Population *p);

/* Checks whether there is only one population left. */
gint only_one_left(Population *p);

/* The main kickoff. */
void run_life(GtkWidget *widget, gpointer data);

/* The step-by-step kickoff. */
gint step_life(GtkWidget *widget, gpointer data);

/* Actually only possible to go back one generation at this moment. */
gint step_life_back(GtkWidget *widget, gpointer data);

/* To stop the simulation during run_life. */
void stop_life(GtkWidget *widget, gpointer data);

/* To clear the playtable. */
void clear_table(GtkWidget *widget, gpointer data);
